clear all
set mem 400m
set mat 800
set more off

** Note: please change the following line to the path of your files
cd "C:\Users\Owner\Desktop\IO Replication Packet\Data Sets\"

use "effect_of_econ MASTER with buffer.dta", clear

rename kad_id id

rename kad_buffer buffer

replace min_dist = 0 if id==1320
replace min_dist = 1 if id==2563
replace min_dist = 0 if id==2385
replace min_dist = 0 if id==2270
replace min_dist = 1 if id==1425
replace min_dist = 1 if id==1480
replace min_dist = 0 if id==1118
replace min_dist = 0 if id==23

replace ecaid=1 if id==66
duplicates drop id, force
drop ECON
gen ECON=0
replace ECON=1 if ecaid==3 | ecaid==2

rename peak_year peak_year2
gen peak_year = 0
replace peak_year = 1 if peak_year2==1 | prior_year==1 

gen ally=0
replace ally=1 if offense==1 | defense==1

gen prop_polity= 0 if pol1~=. | pol2~=. | pol3~=. | pol4~=. | pol5~=.
replace prop_polity=0 if id==1010
replace prop_polity=0 if id==1050
replace prop_polity=0 if id==1345
replace prop_polity=0 if id==1480
replace prop_polity=0 if id==2415
replace prop_polity=0 if id==3030
local i 1
while `i' <=(5) {
replace prop_polity = prop_polity + 1 if pol`i'>=6 & pol`i'~=.
local i = `i' + 1
}

replace prop_polity = prop_polity/N
rename prop_polity jdem

match2 form ECON min_mil jdem min_dist peak_year N buffer prior_neg if year>=1860 & year<=1945 & (offense==1 | defense==1), level(90) m(1) robust(1)

use "matches sensitivity.dta", clear

capture rename  form__00000H control
capture rename  form__00000K treated
capture rename __000001 w
capture rename __00000C km
capture drop __000000
capture rename min_mil__00000K min_mil_treated
capture rename min_mil__00000H min_mil_control
capture rename min_dist__00000K min_dist_treated
capture rename min_dist__00000H min_dist_control
capture rename N__00000K N_treated
capture rename N__00000H N_control
capture rename ally__00000K ally_treated
capture rename ally__00000H ally_control
capture rename peak_year__00000K war_year_treated
capture rename peak_year__00000H war_year_control
capture rename prior_neg__00000K prior_neg_treated
capture rename prior_neg__00000H prior_neg_control
capture rename buffer__00000K buffer_treated
capture rename buffer__00000H buffer_control
capture rename jdem__00000K jdem_treated
capture rename jdem__00000H jdem_control

ttest treated==control

rename min_dist_treated minDt
rename min_dist_control minDc

rename minDt min_dist_treated
rename minDc min_dist_control

*** Rosenbaum bounds sensitivity test
gen delta = treated - control
rbounds delta, gamma(1 (0.1) 5)

*** Balance Statistics
keep  jdem_control jdem_treated prior_neg_treated prior_neg_control N_treated N_control min_mil_treated min_dist_treated min_mil_control min_dist_control war_year_treated  war_year_control buffer_treated buffer_control
save "matches ksmirnov.dta", replace
keep  jdem_treated prior_neg_treated N_treated min_mil_treated min_dist_treated war_year_treated buffer_treated
save "matches ksmirnov 1.dta", replace
use "matches ksmirnov.dta", clear
keep jdem_control prior_neg_control N_control min_mil_control min_dist_control war_year_control buffer_control
append using "matches ksmirnov 1.dta"

gen treat = 0
replace treat = 1 if  min_mil_control ==. |  min_dist_control ==. | N_control==. |  war_year_control==. | buffer_control==.  | prior_neg_control==. | jdem_control==.

replace buffer_treated=0 if buffer_treated==. 
replace buffer_control=0 if buffer_control ==. 
replace war_year_treated=0 if war_year_treated==. 
replace war_year_control=0 if war_year_control ==. 
replace min_mil_control=0 if min_mil_control ==. 
replace min_mil_treated=0 if min_mil_treated ==. 
replace min_dist_control=0 if min_dist_control ==. 
replace min_dist_treated=0 if min_dist_treated ==. 
replace N_control=0 if N_control ==. 
replace N_treated=0 if N_treated ==. 
replace prior_neg_treated=0 if prior_neg_treated==. 
replace prior_neg_control=0 if prior_neg_control ==. 
replace jdem_treated=0 if jdem_treated==. 
replace jdem_control=0 if jdem_control ==. 

gen neg = prior_neg_treated + prior_neg_control
gen buf = buffer_treated + buffer_control
gen war = war_year_treated + war_year_control
gen dist = min_dist_treated + min_dist_control
gen mil = min_mil_treated + min_mil_control
gen n = N_treated + N_control
gen dem = jdem_treated + jdem_control

keep dis mil treat n war buf neg dem
ksmirnov dem, by(treat)
ksmirnov neg, by(treat)
ksmirnov buf, by(treat)
ksmirnov war, by(treat)
ksmirnov n, by(treat)
ksmirnov dis, by(treat)
ksmirnov mil, by(treat)

ranksum dem, by(treat)
ranksum neg, by(treat)
ranksum buf, by(treat)
ranksum war, by(treat)
ranksum n, by(treat)
ranksum dis, by(treat)
ranksum mil, by(treat)

ttest dem, by(treat)
ttest neg, by(treat)
ttest buf, by(treat)
ttest war, by(treat)
ttest n, by(treat)
ttest dis, by(treat)
ttest mil, by(treat)
