clear all
set mem 400m
set more off

** Note: please change the following line to the path of your files
cd "C:\Users\Owner\Desktop\IO Replication Packet\Data Sets\"

use "effect_of_econ MASTER with buffer.dta", clear
rename kad_buffer buffer

rename kad_id id

replace min_dist = 0 if id==1320
replace min_dist = 1 if id==2563
replace min_dist = 0 if id==2385
replace min_dist = 0 if id==2270
replace min_dist = 1 if id==1425
replace min_dist = 1 if id==1480
replace min_dist = 0 if id==1118
replace min_dist = 0 if id==23

replace ecaid=1 if id==66
duplicates drop id, force
drop ECON
gen ECON=0
*replace ECON=1 if ecaid==3 | ecaid==2
replace ECON=1 if ecaid~=0

rename peak_year peak_year2
gen peak_year = 0
replace peak_year = 1 if peak_year2==1 | prior_year==1 

gen ally=0
replace ally=1 if offense==1 | defense==1

gen prop_polity= 0 if pol1~=. | pol2~=. | pol3~=. | pol4~=. | pol5~=.
replace prop_polity=0 if id==1010
replace prop_polity=0 if id==1050
replace prop_polity=0 if id==1345
replace prop_polity=0 if id==1480
replace prop_polity=0 if id==2415
replace prop_polity=0 if id==3030
local i 1
while `i' <=(5) {
replace prop_polity = prop_polity + 1 if pol`i'>=6 & pol`i'~=.
local i = `i' + 1
}

replace prop_polity = prop_polity/N
rename prop_polity jdem


***************************
*MOLINARI BOUNDS ANALYSIS
***************************
*** First Attempt at Computing Molinari Bounds (no assumptions)

*/*use percentage: i.e. allow X % of these to be observed. */
*local p = 1

rename id atopid

keep if year>=1860 /*& prior_neg==0*/ & (offense==1 | defense==1)

* Key Data 1: No sythetic dyads
sum if atopid<1000
local N_y0 = r(N)


sum form if form==1
local N_y1 = r(N)

sum form if ECON==1 & form==1
local N_y1_t1_d1 = r(N)

sum form if ECON==1 & form==0
local N_y0_t1_d1 = r(N)

sum form if ECON==0 & form==1
local N_y1_t0_d1 = r(N)

local N_y0_t0_d0 = `N_y0' - `N_y0_t1_d1'

* Number of failures where treatment is observed and linkage was not offered.
local N_y0_t0_d1 = 0


* Key Probabilities
local denom = `N_y1_t1_d1' + `N_y1_t0_d1' + `N_y0_t1_d1' + `N_y0_t0_d1'
local numer = `N_y1_t1_d1' + `N_y1_t0_d1' 
local p_y1_d1 = `numer'/`denom'
di `p_y1_d1'

local p_y1_d0 =  0 /* Note: This is Q in the Molinari paper */

*sum form if ECON==0
*local n1 = r(N)
*sum form if ECON==0 & form==1
*local n2 = r(N)
*local p_y1_d0 = `n2'/`n1'
*di `p_y1_d0'

*local p_z1_d0 =  /*Note: This is p in the Molinari paper */

local denom1 = `N_y1_t1_d1' + `N_y0_t1_d1'
di `denom1'
local p_y1_d1_z1 = `N_y1_t1_d1'/`denom1'
di `p_y1_d1_z1'

local denom2 = `N_y1_t0_d1' + `N_y0_t0_d1'
di `denom2'
local p_y1_d1_z0 = `N_y1_t0_d1'/`denom2'
di `p_y1_d1_z0'

local denom3 = `N_y1_t1_d1'+`N_y0_t1_d1'+`N_y1_t0_d1'+`N_y0_t0_d1'
local numer3 = `N_y1_t1_d1'+`N_y0_t1_d1'
local p_z1_d1 = `numer3'/`denom3'
di `p_z1_d1'

local denom4 = `N_y1_t1_d1'+`N_y0_t1_d1'+`N_y1_t0_d1'+`N_y0_t0_d1'
local numer4 = `N_y1_t0_d1'+`N_y0_t0_d1'
local p_z0_d1 =  `numer4'/`denom4'
di `p_z0_d1'

local denom = `N_y1_t1_d1' + `N_y1_t0_d1' + `N_y0_t1_d1' + `N_y0_t0_d1' + `N_y0_t0_d0'
local numer = `N_y1_t1_d1' + `N_y1_t0_d1' + `N_y0_t1_d1' + `N_y0_t0_d1' 
local p_d1 = `numer'/`denom'
di `p_d1'

local p_d0 = `N_y0_t0_d0'/`denom'
di `p_d0'


**** KEY RESULTS
* Proposition 1
di `p_y1_d1' - `p_y1_d0'

*** CTE
*Proposition 3 (Lower Bound)
di -`p_d0'-((`p_z1_d1'+`p_y1_d1_z0'*`p_z0_d1'-`p_y1_d1_z1'*`p_z1_d1')*`p_d1')

*Proposition 3 (Upper Bound)
di `p_d0'+((`p_z0_d1'+`p_y1_d1_z1'*`p_z1_d1'-`p_y1_d1_z0'*`p_z0_d1')*`p_d1')

*** CTE (with weak monotonicity)
*Proposition 5
di (`p_y1_d1_z1'-`p_y1_d1_z0')*`p_d1'-`p_d0'

* Proposition 5 (APPENDIX of forthcoming article)
di (`p_y1_d1_z1'-`p_y1_d1_z0')*`p_d1'+`p_d0'


** MANSKI BOUNDS
sum if atopid<1000
local N_y0 = r(N)
sum form if form==1
local N_y1 = r(N)
sum form if ECON==1 & form==1
local N_y1_t1 = r(N)
sum form if ECON==1 & form==0
local N_y0_t1 = r(N)
sum form if ECON==0 & form==1
local N_y1_t0 = r(N)
sum form if ECON==0 & form==0
local N_y0_t0 = r(N)
sum form if ECON==1
local N_t1 = r(N)
sum form if ECON==0
local N_t0 = r(N)

local p_y1_z1 = `N_y1_t1'/(`N_y1_t1' + `N_y0_t1')

local p_y1_z0 = `N_y1_t0'/(`N_y1_t0' + `N_y0_t0')

local p_y0_z0 = `N_y0_t0'/(`N_y1_t0' + `N_y0_t0')

local p_z1 = `N_t1'/(`N_y0' + `N_y1')

local p_z0 = `N_t0'/(`N_y0' + `N_y1')

* lower bound
di `p_y1_z1'*`p_z1' - `p_y0_z0'*`p_z0' - `p_z1'

* upper bound
di `p_y1_z1'*`p_z1' + `p_z0' - `p_y0_z0'*`p_z0'
